# Ilanot Backend / Data integration

This program is part of the Ilanot portal. The purpose of this program is to integrate various data sources and controlling the data and process flow. It retrieves the data and processes them with the tools for further usage of the portal.

# What it does

* login to the TextGrid Ilanot project
* download the index file which contains references to the corresponding resources (currently it is not possible to automatically determine matching items)
* for each manuscript
  * Check if the files linked in the index file have changed, if yes then
  * download TEI transcriptions
  * download TRIVIA annotations
  * dowmload image of manuscript
  * process TEI and TRIVIA resources with [Ilanot RDF converter](https://gitlab.gwdg.de/ilanot/ilanot-conversion-to-rdf)
  * clear and update Fuseki store with the generated RDF tripels from the RDF converter
  * process the image with [VIPS](https://github.com/libvips/libvips) as multilayered TIFF
  * copies the multilayered TIFF to a readable directory accessible by the IIIF server
* update the status XML status file for the next run   

# Building

This program needs [Java JDK](https://openjdk.java.net/) version 11 or greater and [Maven 3](https://maven.apache.org/index.html) as build tool. If both programs are installed and available on the search path (check if java -version and mvn -version works). If this works, then simply execute

```
cd directory-of-iltextgrid2backend
mvn
```

This repository contains three main branches. MASTER which holds the production version, staging for release candidates (extended testing) and dev for the development branch. On this Gitlab the project is setup so it is automatically build with CI. The releases are pushed to the FE APT server and then installed via Puppet. To determine which branch goes to which server - there is a dev, staging and production server corresponing to the branches - the build name changes. The different builds are activated through different maven profiles

## Development build (default)
```
mvn
```
or
```
mvn -D env=dev
```
For building ilanot-data-integration-dev packages.
## Staging build
```
mvn -D env=staging
```
For building ilanot-data-integration-staging packages.
## Production build
```
mvn -D env=prod
```
For building ilanot-data-integration packages.
# Configuration

See application.properties.samples. Copy the file to application.properties and change

| Parameter             | Explanation                                                                       | Example / Default             |
| --------------------- |-----------------------------------------------------------------------------------| -------------------|
| tg.username           | username to login to textgrid project                                             | johndoe |
| tg.password           | password to login to textgrid project                                             | letmein |
| tg.url                | url for authentication, normally no need to change                                | https://textgridlab.org/1.0/WebAuthN |
| tg.authinstance       | auth instance for authentication , no need to change                              | textgrid-esx2.gwdg.de |
| ilan.indexurl         | enter a TexGrid URI which contains the index file (see structure below)           | textgrid:3qr823
| output.dir            | path o a temporary directory where this tool can write files, dir must exist      | /var/ilanot/tmp |
| output.dir.purge      | set it to true or false to delete temporary files after running                   | false |
| output.iiif           | directory where the IIIF server serves images from, this program needs write privileges on this folder | /opt/nginx/images |
| command.rdfconverter  | full path to to make_rdf.pl [RDF Converter](https://gitlab.gwdg.de/ilanot/ilanot-conversion-to-rdf)  | /opt/ilanot-rdf/make_rdf.pl |
| command.vips          | full path to vips command for preparing pyramidal tiff files                      | /usr/bin/vips
| command.vipsOptions   | options on how vips should convert the images. *Note:* leave $source and $output as placeholder in this line! For options see VIPS documentation or [IPImage](https://iipimage.sourceforge.io/documentation/images/)                     | tiffsave $source $output --tile --pyramid --compression jpeg  --Q 92  --tile-width 256 --tile-height 256 | 
|fuseki.url             | URL to fuseki endpoint including name of dataset (MUST exist) to write and delete RDF triples|http://localhost:8080/fuseki/ds|
| logging.level.de.sub.goettingen.ilanot.iltextgrid2backend           | Log level, see logging section                                            | INFO |
| logging.file.name           | Path to log file (optional parameter), see logging                                             | /tmp/log.txt |
| textgrid.log.upload      | set it to true or false to upload log of this application to TextGrid Rep, logging must be enabled!                   | false |
| textgrid.log.resource    | URI for a textgrid resource that should hold log data, see logging section                                             | textgrid:3r0982 |
| triviaIds.compile      | set it to true or false to generate a list of trivia IDs from the files, only useful as aid for editors | false |
| triviaIds.tgId    | URI for a textgrid resource that should hold a list of trivia IDs. Only valid if triviaIds.compile is set to true. If empty the list will be written to stdout.     | textgrid:3r0100 |
| ontology.include           | set to true to load the TTL ontology into Fuseki, false disables this.      | true |
| ontology.loadGit           | set to true to load the TTL from a GIT repository. Can be loaded from file if set to false (see ontology.path). | false |
| ontology.gitUrl            | full git url which contains the repository of the ontology    | https://gitlab.gwdg.de/project/myrepo.git |
| ontology.gitBranch       | which remote branch should be cloned. Must be full path, see example for master branch | refs/heads/master |
| ontology.gitToken         | if the GIT repository is private, than an access token can be created on the project setting. You can enter the token here. If no token is specified, it clones it as anonymous and the repo must be public. | SDsd34djfhsdfjk |
| ontology.path           | if ontology.loadGit is set to false then a full path to a .ttl file which contains the ontology can be specified. Leave empty if loadGit is set to true.    | /home/work/ontology/ilanot.ttl |
| json.output         | set to true if you want a json output of the manuscript structure including references to TEI. TRIVIA and image files | false |
| json.output.path           | if json.output is true, then you must specify a full output path including file name for the JSON structure.   | /tmp/structure.json |


## Logging configuration

By default this application logs info, warning and error messages to the console (stdout). If you want to change the log level, add  the property `logging.level.de.sub.goettingen.ilanot.iltextgrid2backend` to the `application.properties`. E.g.
```
logging.level.de.sub.goettingen.ilanot.iltextgrid2backend=TRACE
```

Available Levels: TRACE, DEBUG, INFO, WARN, ERROR, FATAL, OFF

Logging to a file: Add the property `logging.file.name` to `application.properties` and set the full or relative path to a file. E.g.
```
logging.file.name=/var/log/iltgsync.log
```

### Turn off console logging

Add `logging.pattern.console=` to `application.properties`. *Note* it makes sense that you turn on file logging instead.

### Log file size and overwriting

By default the log4j system has a maximum log file size of 10 MB and will rotate the log file, but does not delete it. To change this behavior, add for example these entries to `application.properties`
```
logging.file.max-history=7
logging.file.max-size=1MB
```

Max-history sets the number of days to keep and max-size limits the file size. If the application logs 24 MB per day this means that it will create 24 files per day.

### Upload Log data to TextGrid Rep

It is possible to upload the log data to a TextGrid project. The parameter `logging.file.name` must be set, `textgrid.log.upload` must be set to true and the resource `textgrid.log.resource` must exist and set to to TextGrid URI without a revision.

Please go to the TextGrid project, click file -> new -> element -> plain text. Give it a name, associate it to an aggregation (optional), right click on the item -> copy URI. Insert the URI into the configuration, but delete everything after the ".". E.g. textgrid:3vpbc.0 -> textgrid:3vpbc.

# Index file structure

```xml
<?xml version="1.0" encoding="UTF-8"?>
<ilanot>
    <manuscript name="test" uri="textgrid:abc">
        <imageAggregation uri="textgrid:def">
            <file name="nice image" uri="textgrid:3697"/>
        </imageAggregation>
        <editionAggregation name="tei files" uri="textgrid:984">
            <file name="TEI1" uri="textgrid:34"/>
            <file name="TEI2" uri="textgrid:42"/>
        </editionAggregation>
        <triviaAggregation uri="textgrid:sadas">
            <file name="trivia 1" uri="textgrid:34534"/>
            <file name="trivia 2" uri="textgrid:sdfsdf"/>
        </triviaAggregation>
    </manuscript>
</ilanot>
```

This index file with this structure is necessary on the TextGrid project.

# Running

The application will search for a application.proerties files in the current directory. Make sure that the file exists beneath the Jar file. Then simply execute it with

```
java -jar il-textgrid2backend-VERSION-SNAPSHOT.jar
```

Replace VERSION with the correct version. If the application can't find the configuration, try

```
java -jar il-textgrid2backend-VERSION-SNAPSHOT.jar --spring.config.location=file:///etc/ilanot/application.properties
```

file:// needs to be present. Replace /etc/ilanot/application.properties with the correct folder

## Reprocessing old synchronisation

This is only possible if output.dir.purge is set to false. The manual reprocessing allows to load the created Turtle files into Fuseki. 
This helps if the database is almost empty due wrong XML files, e.g. typo and only one file is been created by this tool.
To load a previous synchrinsation run it like described in running, but with the following syntax

```
java -jar il-textgrid2backend-VERSION-SNAPSHOT.jar --reprocess NAMEOFDIRECTORY
```

Replace NAMEOFDIRECTORY with one of the sub directory from output.dir. E.g.:

```
java -jar il-textgrid2backend-VERSION-SNAPSHOT.jar --reprocess 20200818131449222
```

Replace VERSION with the correct version of this tool. 

# Used Frameworks

- Spring framework with [Spring Boot](https://spring.io/projects/spring-boot) as console application together with the provied libraries (log4j etc.)
- JDOM2 for XML functions [JDOM](http://www.jdom.org/)
- [Apache Jena](https://jena.apache.org/) for accessing Fuseki 

# Author

Johannes Biermann, Abteilung Forschung & Entwicklung, Niedersächsische. Staats- und Universitaetsbibliothek Göttingen (biermann@sub.uni-goettingen.de) 

# Revision history

* 26.06.2020 first version
* 30.06.2020 added TG Rep log upload functionality, added fuseki connection test
* 18.08.2020 more checks before loading fuseki, reprocess function
* 20.10.2020 fixed handling of changes of a manuscript (everything needs to be reprocessed)
* 26.10.2020 added the option to compile a list of trivia resources for the editors
* 09.11.2020 added loading of ontology
* 09.11.2020 package rename, other file size check
* 10.11.2020 fixed NPE and empty trivia id list
* 13.01.2021 Multithreading support for much better peformance
* 26.04.2021 Added JSON output support


# License 

Apache 2.0.